/*
	Copyright 2016-2018 Marco "eukara" Hladik
	
	MIT LICENSE

	Permission is hereby granted, free of charge, to any person 
	obtaining a copy of this software and associated documentation 
	files (the "Software"), to deal in the Software without 
	restriction, including without limitation the rights to use,
	copy, modify, merge, publish, distribute, sublicense, and/or
	sell copies of the Software, and to permit persons to whom the
	Software is furnished to do so, subject to the following conditions:

	The above copyright notice and this permission notice shall be
	included in all copies or substantial portions of the Software.

	THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
	EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
	OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
	NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
	HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
	WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
	FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
	OTHER DEALINGS IN THE SOFTWARE.
*/

/*
=================
Timer_Begin

Initiates a new state timer
=================
*/
void Timer_Begin( float fTime, float fMode) {
	if ( fMode == GAME_FREEZE ) {
		fGameState = GAME_FREEZE;
	} else if ( fMode == GAME_ACTIVE ) {
		fGameState = GAME_ACTIVE;
	} else if ( fMode == GAME_END ) {
		fGameState = GAME_END;
	} else if ( fMode == GAME_COMMENCING ) {
		fGameState = GAME_COMMENCING;
	} else if ( fMode == GAME_OVER ) {
		fGameState = GAME_OVER;
	}
	
	fGameTime = fTime;
}

/*
=================
Timer_Update

Called once every frame to check the status of things
=================
*/
void Timer_Update( void ) {
	static float fVoxTimer;
	
	// This map has been played enough we think
	if ( fGameState != GAME_OVER ) {
		if ( cvar( "mp_timelimit" ) > 0 ) {
			if ( autocvar_sv_voxannounce == TRUE ) {
				if ( fVoxTimer > time ) {
					return;
				}
				
				float fTimeLeft = ( cvar( "mp_timelimit" ) * 60 ) - time;
				for ( int i = 0; i <= 10; i++ ) {
					if ( rint( fTimeLeft ) == ( i * 60 ) ) {
						Vox_Broadcast( sprintf( "%s minutes remaining", Vox_TimeToString( fTimeLeft / 60 ) ) );
						fVoxTimer = time + 10.0f;
					}
				}
			}
		
			if ( time >= ( cvar( "mp_timelimit" ) * 60 ) ) {
				Timer_Begin( 5, GAME_OVER );
			}
		}
	}
	
	if ( ( fGameState == GAME_OVER ) && ( fGameTime < 0 ) ) {
		for ( int i = 0; i < iMapCycleCount; i++ ) {
			if ( sMapCycle[ i ] == mapname ) {
				if ( ( i + 1 ) < iMapCycleCount ) {
					localcmd( sprintf( "changelevel %s\n", sMapCycle[ i + 1 ] ) );
					return;
				} else {
					localcmd( sprintf( "changelevel %s\n", sMapCycle[ 0 ] ) );
				}
			}
		}
	}
	
	// Okay, this means that timelimit is not the only deciding factor
	if ( autocvar_mp_winlimit > 0 && fGameState != GAME_OVER ) {
		// It really doesn't matter who won. Do some logging perhaps?
		if ( iWon_CT == autocvar_mp_winlimit ) {
			Timer_Begin( 5, GAME_OVER );
		} else if ( iWon_T == autocvar_mp_winlimit ) {
			Timer_Begin( 5, GAME_OVER );
		}
	}
	
	if ( fGameState == GAME_INACTIVE ) {
		return;
	}
	
	fGameTime -= frametime;
	
	if ( fGameState == GAME_COMMENCING || fGameState == GAME_END ) {
		if ( fGameTime <= 0 ) {
			if ( iWon_T == 0 && iWon_CT == 0 ) {
				Money_ResetTeamReward();
				Rules_Restart( TRUE );
			} else {
				if ( autocvar_mp_halftime == TRUE && ( autocvar_mp_winlimit / 2 == iRounds ) ) {
					Money_ResetTeamReward();
					Rules_SwitchTeams();
					Rules_Restart( TRUE );
				} else {
					Rules_Restart( FALSE );
				}
			}
		}
		return;
	}
	
	if ( ( fGameState == GAME_ACTIVE ) || ( fGameState == GAME_FREEZE ) ) {
		if ( fGameTime <= 0 ) {
			if ( fGameState == GAME_ACTIVE ) {
				Rules_TimeOver();
				Timer_Begin( 5, GAME_END ); // Round is over, 5 seconds til a new round starts
			} else {
				Timer_Begin( autocvar_mp_roundtime * 60, GAME_ACTIVE ); // Unfreeze
				Radio_StartMessage();	
			}
		}
	}
}
